@@ -6,13 +6,14 @@
 package Lemonldap::NG::Portal::Menu;
 
 use strict;
+use utf8;
 use warnings;
 use Lemonldap::NG::Portal::Simple;
 use Lemonldap::NG::Portal::_LibAccess;
 use base qw(Lemonldap::NG::Portal::_LibAccess);
 use Clone qw(clone);
 
-our $VERSION  = '1.3.0';
+our $VERSION = '1.9.1';
 our $catlevel = 0;
 
 ## @method void menuInit()
@@ -35,7 +36,8 @@ sub menuInit {
 
     # Try to change password
     $self->{menuError} =
-      $self->_subProcess(qw(passwordDBInit modifyPassword sendPasswordMail))
+      $self->_subProcess(
+        qw(passwordDBInit modifyPassword passwordDBFinish sendPasswordMail))
       unless $self->{ignorePasswordChange};
 
     # Default menu error code
@@ -57,6 +59,7 @@ sub menuInit {
             scalar(
                 grep { $_ == $self->{menuError} } (
                     25,    #PE_PP_CHANGE_AFTER_RESET
+                    26,    #PE_PP_PASSWORD_MOD_NOT_ALLOWED
                     27,    #PE_PP_MUST_SUPPLY_OLD_PASSWORD
                     28,    #PE_PP_INSUFFICIENT_PASSWORD_QUALITY
                     29,    #PE_PP_PASSWORD_TOO_SHORT
@@ -124,7 +127,7 @@ sub displayModules {
 # Returns categories and applications list as HTML::Template loop
 # @return categories and applications list
 sub appslist {
-    my ($self) = splice @_;
+    my ($self) = @_;
     my $appslist = [];
 
     return $appslist unless defined $self->{applicationList};
@@ -149,7 +152,7 @@ sub appslist {
 # @param catlevel Category level
 # @return Category Hash
 sub _buildCategoryHash {
-    my ( $self, $catid, $cathash, $catlevel ) = splice @_;
+    my ( $self, $catid, $cathash, $catlevel ) = @_;
     my $catname = $cathash->{catname} || $catid;
     my $applications;
     my $categories;
@@ -198,7 +201,7 @@ sub _buildCategoryHash {
 # @param $apphash Hash of application elements
 # @return Application Hash
 sub _buildApplicationHash {
-    my ( $self, $appid, $apphash ) = splice @_;
+    my ( $self, $appid, $apphash ) = @_;
     my $applications;
 
     # Get application items
@@ -284,7 +287,7 @@ sub appslistDescription {
 # @param catlevel Category level
 # @return HTML string
 sub _displayConfCategory {
-    my ( $self, $catname, $cathash, $catlevel ) = splice @_;
+    my ( $self, $catname, $cathash, $catlevel ) = @_;
     my $html;
     my $key;
 
@@ -425,7 +428,7 @@ sub _displayConfDescription {
 # @param $apphash Menu elements
 # @return filtered hash
 sub _filter {
-    my ( $self, $apphash ) = splice @_;
+    my ( $self, $apphash ) = @_;
     my $filteredHash;
     my $key;
 
@@ -600,11 +603,11 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2008, 2009, 2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2008-2010 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
 =item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2008-2016 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
 =back
 
