@@ -71,7 +71,7 @@ use Digest::MD5;
 #inherits Apache::Session
 #link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
 
-our $VERSION = '1.4.6';
+our $VERSION = '1.9.1';
 
 use base qw(Lemonldap::NG::Common::CGI Exporter);
 our @ISA;
@@ -184,6 +184,14 @@ use constant {
     PM_ERROR_MSG             => 21,
     PM_LAST_LOGINS           => 22,
     PM_LAST_FAILED_LOGINS    => 23,
+    PM_OIDC_CONSENT          => 24,
+    PM_OIDC_SCOPE_OPENID     => 25,
+    PM_OIDC_SCOPE_PROFILE    => 26,
+    PM_OIDC_SCOPE_EMAIL      => 27,
+    PM_OIDC_SCOPE_ADDRESS    => 28,
+    PM_OIDC_SCOPE_PHONE      => 29,
+    PM_OIDC_SCOPE_OTHER      => 30,
+    PM_OIDC_CONFIRM_LOGOUT   => 31,
 };
 
 # EXPORTER PARAMETERS
@@ -213,7 +221,9 @@ our @EXPORT = qw( PE_IMG_NOK PE_IMG_OK PE_INFO PE_REDIRECT PE_DONE PE_OK
   PM_SAML_IDPSELECT PM_SAML_IDPCHOOSEN PM_REMEMBERCHOICE PM_SAML_SPLOGOUT
   PM_REDIRECTION PM_BACKTOSP PM_BACKTOCASURL PM_LOGOUT PM_OPENID_EXCHANGE
   PM_CDC_WRITER PM_OPENID_RPNS PM_OPENID_PA PM_OPENID_AP PM_ERROR_MSG
-  PM_LAST_LOGINS PM_LAST_FAILED_LOGINS
+  PM_LAST_LOGINS PM_LAST_FAILED_LOGINS PM_OIDC_CONSENT PM_OIDC_SCOPE_OPENID
+  PM_OIDC_SCOPE_PROFILE PM_OIDC_SCOPE_EMAIL PM_OIDC_SCOPE_ADDRESS
+  PM_OIDC_SCOPE_PHONE PM_OIDC_SCOPE_OTHER PM_OIDC_CONFIRM_LOGOUT
 );
 our %EXPORT_TAGS = ( 'all' => [ @EXPORT, 'import' ], );
 
@@ -249,6 +259,8 @@ sub new {
       or $self->abort( "Configuration error",
         "Unable to get configuration: $Lemonldap::NG::Common::Conf::msg" );
 
+    $self->{multiValuesSeparator} ||= ';';
+
     # Test mandatory elements
 
     # 1. Sessions backend
@@ -286,6 +298,12 @@ sub new {
         $self->{captchaStorageOptions} = $self->{globalStorageOptions};
     }
 
+    # OpenIDConnect
+    $self->{oidcStorage} ||= $self->{globalStorage};
+    if ( !$self->{oidcStorageOptions} or !%{ $self->{oidcStorageOptions} } ) {
+        $self->{oidcStorageOptions} = $self->{globalStorageOptions};
+    }
+
     # 2. Domain
     $self->abort( "Configuration error",
         "You've to indicate a domain for cookies" )
@@ -307,7 +325,8 @@ sub new {
     foreach my $type (qw(authentication userDB passwordDB registerDB)) {
         my $module_name = 'Lemonldap::NG::Portal::';
         my $db_type     = $type;
-        my $db_name     = $self->{$db_type};
+        my $db_name     = $self->{$db_type}
+          or $self->abort("'$db_type' is not set");
 
         # Adapt module type to real module name
         $db_type =~ s/authentication/Auth/;
@@ -335,7 +354,7 @@ sub new {
     }
 
     # Check issuerDB path to load the correct issuerDB module
-    foreach my $issuerDBtype (qw(SAML OpenID CAS)) {
+    foreach my $issuerDBtype (qw(SAML OpenID CAS OpenIDConnect)) {
         my $module_name = 'Lemonldap::NG::Portal::IssuerDB' . $issuerDBtype;
 
         $self->lmLog( "[IssuerDB activation] Try issuerDB module $issuerDBtype",
@@ -483,7 +502,7 @@ sub new {
 # @param ignoreError set to 1 if error should not appear in logs
 # @return boolean
 sub loadModule {
-    my ( $self, $module, $ignoreError ) = splice @_;
+    my ( $self, $module, $ignoreError ) = @_;
 
     return 1 unless $module;
 
@@ -528,7 +547,7 @@ sub getConf {
 # @param base64 Encode value in base64
 # @return nothing
 sub setHiddenFormValue {
-    my ( $self, $key, $val, $prefix, $base64 ) = splice @_;
+    my ( $self, $key, $val, $prefix, $base64 ) = @_;
 
     # Default values
     $prefix = "lmhidden_" unless defined $prefix;
@@ -539,6 +558,7 @@ sub setHiddenFormValue {
         $key = $prefix . $key;
         $val = encode_base64($val) if $base64;
         $self->{portalHiddenFormValues}->{$key} = $val;
+        $self->lmLog( "Store $val in hidden key $key", 'debug' );
     }
 }
 
@@ -549,7 +569,7 @@ sub setHiddenFormValue {
 # @param base64 Decode value from base64
 # @return string The associated value
 sub getHiddenFormValue {
-    my ( $self, $key, $prefix, $base64 ) = splice @_;
+    my ( $self, $key, $prefix, $base64 ) = @_;
 
     # Default values
     $prefix = "lmhidden_" unless defined $prefix;
@@ -561,6 +581,7 @@ sub getHiddenFormValue {
     if ( my $val = $self->param($key) ) {
         $val = decode_base64($val) if $base64;
         return $val;
+        $self->lmLog( "Hidden value $val found for key $key", 'debug' );
     }
 
     # No value found
@@ -573,13 +594,17 @@ sub getHiddenFormValue {
 # @param keys Array reference of keys
 # @return nothing
 sub clearHiddenFormValue {
-    my ( $self, $keys ) = splice @_;
+    my ( $self, $keys ) = @_;
 
     unless ( defined $keys ) {
         delete $self->{portalHiddenFormValues};
+        $self->lmLog( "Delete all hidden values", 'debug' );
     }
     else {
-        delete $self->{portalHiddenFormValues}->{$_} foreach (@$keys);
+        foreach (@$keys) {
+            delete $self->{portalHiddenFormValues}->{$_};
+            $self->lmLog( "Delete hidden value for key $_", 'debug' );
+        }
     }
 
     return;
@@ -637,7 +662,7 @@ sub initCaptcha {
 # @param captcha code generated by Authen::Captcha
 # @return a constant
 sub checkCaptcha {
-    my ( $self, $code, $ccode ) = splice @_;
+    my ( $self, $code, $ccode ) = @_;
 
     # Get captcha object
     my $captcha = Lemonldap::NG::Common::Captcha->new(
@@ -667,7 +692,7 @@ sub checkCaptcha {
 # @param captcha code generated by Authen::Captcha
 # @return a constant
 sub removeCaptcha {
-    my ( $self, $ccode ) = splice @_;
+    my ( $self, $ccode ) = @_;
 
     # Get captcha object
     my $captcha = Lemonldap::NG::Common::Captcha->new(
@@ -696,7 +721,7 @@ sub removeCaptcha {
 # @param value Parameter value
 # @return 1 if url can be trusted, 0 else
 sub isTrustedUrl {
-    my ( $self, $url ) = splice @_;
+    my ( $self, $url ) = @_;
     return
          $url =~ m#^https?://$self->{reVHosts}(:\d+)?/#o
       || $self->{trustedDomains} eq "*"
@@ -710,7 +735,7 @@ sub isTrustedUrl {
 # @param value Parameter value
 # @return 1 if attack detected, 0 else
 sub checkXSSAttack {
-    my ( $self, $name, $value ) = splice @_;
+    my ( $self, $name, $value ) = @_;
 
     # Empty values are not bad
     return 0 unless $value;
@@ -742,7 +767,31 @@ _RETURN $string Error string
 sub msg {
     my $self = shift;
     my $code = shift;
-    return &Lemonldap::NG::Portal::_i18n::msg( $code, $self->{lang} );
+    my $msg;
+
+    # Check for customized message
+    foreach ( @{ $self->{lang} } ) {
+        if ( $self->{ "msg_" . $_ . "_" . $code } ) {
+            $msg = $self->{ "msg_" . $_ . "_" . $code };
+            last;
+        }
+    }
+    $msg ||= $self->{ "msg_" . $code };
+
+    # Use customized message or built-in message
+    if ( defined $msg ) {
+
+        # Manage UTF-8
+        utf8::decode($msg);
+
+        $self->lmLog( "Use customized message $msg for message $code",
+            'debug' );
+    }
+    else {
+        $msg = &Lemonldap::NG::Portal::_i18n::msg( $code, $self->{lang} );
+    }
+
+    return $msg;
 }
 
 ##@method string error(int code)
@@ -778,8 +827,6 @@ sub error {
         $msg = &Lemonldap::NG::Portal::_i18n::error( $code, $self->{lang} );
     }
 
-    # Return message
-    # Manage SOAP
     return $msg;
 }
 
@@ -850,14 +897,15 @@ sub redirect {
     }
 }
 
-## @method protected hashref getApacheSession(string id, boolean noInfo)
+## @method protected hashref getApacheSession(string id, boolean noInfo, boolean $force)
 # Try to recover the session corresponding to id and return session datas.
-# If $id is set to undef, return a new session.
+# If $id is set to undef or if $force is true, return a new session.
 # @param id session reference
 # @param noInfo do not set Apache REMOTE_USER
+# @param force Force session creation if it does not exist
 # return Lemonldap::NG::Common::Session object
 sub getApacheSession {
-    my ( $self, $id, $noInfo ) = @_;
+    my ( $self, $id, $noInfo, $force ) = @_;
 
     my $apacheSession = Lemonldap::NG::Common::Session->new(
         {
@@ -866,6 +914,7 @@ sub getApacheSession {
             cacheModule          => $self->{localSessionStorage},
             cacheModuleOptions   => $self->{localSessionStorageOptions},
             id                   => $id,
+            force                => $force,
             kind                 => "SSO",
         }
     );
@@ -875,6 +924,11 @@ sub getApacheSession {
         return;
     }
 
+    if ( $id and !$force and !$apacheSession->data ) {
+        $self->lmLog( "Session $id not found", 'debug' );
+        return;
+    }
+
     unless ($noInfo) {
         $self->setApacheUser( $apacheSession->data->{ $self->{whatToTrace} } )
           if ($id);
@@ -883,13 +937,17 @@ sub getApacheSession {
     return $apacheSession;
 }
 
-## @method protected hashref getPersistentSession(string id)
-# Try to recover the persitent session corresponding to id and return session datas.
-# If $id is set to undef, return a new session.
-# @param id session reference
+## @method protected hashref getPersistentSession(string uid)
+# Try to recover the persistent session corresponding to uid and return session datas.
+# @param uid main user identifier (whatToTrace)
 # return Lemonldap::NG::Common::Session object
 sub getPersistentSession {
-    my ( $self, $id ) = splice @_;
+    my ( $self, $uid ) = @_;
+
+    return unless defined $uid;
+
+    # Compute persistent identifier
+    my $pid = $self->_md5hash($uid);
 
     my $persistentSession = Lemonldap::NG::Common::Session->new(
         {
@@ -897,7 +955,7 @@ sub getPersistentSession {
             storageModuleOptions => $self->{persistentStorageOptions},
             cacheModule          => $self->{localSessionStorage},
             cacheModuleOptions   => $self->{localSessionStorageOptions},
-            id                   => $id,
+            id                   => $pid,
             force                => 1,
             kind                 => "Persistent",
         }
@@ -907,6 +965,16 @@ sub getPersistentSession {
         $self->lmLog( $persistentSession->error, 'debug' );
     }
 
+    # Set _session_uid if not already present
+    unless ( defined $persistentSession->data->{_session_uid} ) {
+        $persistentSession->update( { '_session_uid' => $uid } );
+    }
+
+    # Set _utime if not already present
+    unless ( defined $persistentSession->data->{_utime} ) {
+        $persistentSession->update( { '_utime' => time } );
+    }
+
     return $persistentSession;
 }
 
@@ -915,7 +983,7 @@ sub getPersistentSession {
 # @param $s String to hash
 # @return hashed value
 sub _md5hash {
-    my ( $self, $s ) = splice @_;
+    my ( $self, $s ) = @_;
     return substr( Digest::MD5::md5_hex($s), 0, 32 );
 }
 
@@ -928,7 +996,7 @@ sub _md5hash {
 # @param id optional SSO session ID
 # @return nothing
 sub updatePersistentSession {
-    my ( $self, $infos, $uid, $id ) = splice @_;
+    my ( $self, $infos, $uid, $id ) = @_;
 
     # Return if no infos to update
     return () unless ( ref $infos eq 'HASH' and %$infos );
@@ -939,8 +1007,7 @@ sub updatePersistentSession {
     $uid ||= $self->{sessionInfo}->{ $self->{whatToTrace} };
     return () unless ($uid);
 
-    my $persistentSession =
-      $self->getPersistentSession( $self->_md5hash($uid) );
+    my $persistentSession = $self->getPersistentSession($uid);
 
     $persistentSession->update($infos);
 
@@ -963,7 +1030,7 @@ sub updatePersistentSession {
 # @param id Session ID
 # @return nothing
 sub updateSession {
-    my ( $self, $infos, $id ) = splice @_;
+    my ( $self, $infos, $id ) = @_;
 
     # Return if no infos to update
     return () unless ( ref $infos eq 'HASH' and %$infos );
@@ -1010,7 +1077,7 @@ sub updateSession {
 # @param value Value to add
 # @param id optional Session identifier
 sub addSessionValue {
-    my ( $self, $key, $value, $id ) = splice @_;
+    my ( $self, $key, $value, $id ) = @_;
 
     # Mandatory parameters
     return () unless defined $key;
@@ -1048,7 +1115,7 @@ sub addSessionValue {
 # @param value the complete value
 # @return first value
 sub getFirstValue {
-    my ( $self, $value ) = splice @_;
+    my ( $self, $value ) = @_;
 
     my @values = split /\Q$self->{multiValuesSeparator}\E/, $value;
 
@@ -1077,11 +1144,11 @@ sub _subProcess {
 # status module with the result (portal error).
 sub updateStatus {
     my $self = shift;
-    print $Lemonldap::NG::Handler::Simple::statusPipe (
-        $self->{user} ? $self->{user} : $self->ipAddr )
+    my $p    = $Lemonldap::NG::Handler::Main::tsv->{statusPipe};
+    print $p ( $self->{user} ? $self->{user} : $self->ipAddr )
       . " => $ENV{SERVER_NAME}$ENV{SCRIPT_NAME} "
       . $self->{error} . "\n"
-      if ($Lemonldap::NG::Handler::Simple::statusPipe);
+      if ($p);
 }
 
 ##@method protected string notification()
@@ -1125,7 +1192,7 @@ sub get_user {
 # @param type auth/user/password/issuer
 # @return module name
 sub get_module {
-    my ( $self, $type ) = splice @_;
+    my ( $self, $type ) = @_;
 
     if ( $type =~ /auth/i ) {
         if ( defined $self->{_multi}->{stack}->[0] ) {
@@ -1328,7 +1395,9 @@ sub printImage {
         return;
     }
     print $self->header(
-        $type . '; charset=utf-8; content-length=' . ( stat($file) )[10] );
+        -type             => "$type; charset=utf-8",
+        '-Content-Length' => ( stat($file) )[10]
+    );
     my $buffer = "";
     while ( read( IMAGE, $buffer, 4096 ) ) {
         print $buffer;
@@ -1346,7 +1415,7 @@ sub stamp {
 # @param $sec number of seconds
 # @return a formated time
 sub convertSec {
-    my ( $self, $sec ) = splice @_;
+    my ( $self, $sec ) = @_;
     my ( $day, $hrs, $min ) = ( 0, 0, 0 );
 
     # Calculate the minutes
@@ -1375,7 +1444,7 @@ sub convertSec {
 # Return skin name
 # @return skin name
 sub getSkin {
-    my ($self) = splice @_;
+    my ($self) = @_;
 
     my $skin = $self->{portalSkin};
 
@@ -1460,6 +1529,23 @@ sub process {
           issuerForAuthUser autoRedirect)
     );
     $self->updateStatus;
+    if (   !$self->{noAjaxHook}
+        and $self->http('Accept') =~ m#(?:application|text)/json# )
+    {
+        if ( ( my $code = $self->{error} ) > 0 ) {
+            print $self->header(
+                -status                        => '401 Unauthorizated',
+                '-WWW-Authenticate'            => "SSO $self->{portal}",
+                '-Access-Control-Allow-Origin' => '*',
+            );
+            $self->quit;
+        }
+        else {
+            $self->header( -type => 'application/json' );
+            print '{"result":1,"message":"Authenticated"}';
+            $self->quit;
+        }
+    }
     return ( ( $self->{error} > 0 ) ? 0 : 1 );
 }
 
@@ -1598,9 +1684,10 @@ sub controlExistingSession {
 
         if ( $captcha && $captcha->image ) {
             binmode STDOUT;
-            print $self->header( 'image/png'
-                  . '; charset=utf-8; content-length='
-                  . length( $captcha->image ) );
+            print $self->header(
+                -type             => 'image/png; charset=utf-8',
+                '-Content-Length' => length( $captcha->image )
+            );
             print $captcha->image;
         }
         $self->quit();
@@ -1704,7 +1791,8 @@ sub controlExistingSession {
                 return $self->{error} if $self->{error} > 0;
 
                 # Collect logout services and build hidden iFrames
-                if ( %{ $self->{logoutServices} } ) {
+                if ( $self->{logoutServices} and %{ $self->{logoutServices} } )
+                {
 
                     $self->lmLog(
                         "Create iFrames to forward logout to services",
@@ -1777,6 +1865,7 @@ sub controlExistingSession {
             # A session has been found => call existingSession
             my $r = $self->_sub( 'existingSession', $id, $self->{sessionInfo} );
             if ( $r == PE_DONE ) {
+                $self->{user}  = $self->{sessionInfo}->{_whatToTrace};
                 $self->{error} = $self->_subProcess(
                     qw(checkNotification issuerDBInit authInit issuerForAuthUser authFinish autoRedirect)
                 );
@@ -1946,6 +2035,7 @@ sub modifyPassword {
 
 ##@apmethod int setSessionInfo()
 # Set ipAddr, startTime, updateTime, _utime and _userDB
+# Set _lastSeen if activity timeout is configured
 # Call setSessionInfo() in UserDB* module
 #@return Lemonldap::NG::Portal constant
 sub setSessionInfo {
@@ -1966,6 +2056,7 @@ sub setSessionInfo {
         $self->{sessionInfo}->{_utime} ||= time();
         $self->{sessionInfo}->{startTime} =
           strftime( "%Y%m%d%H%M%S", localtime() );
+        $self->{sessionInfo}->{_lastSeen} = time() if $self->{timeoutActivity};
     }
 
     # Get environment variables matching exportedVars
@@ -2037,16 +2128,14 @@ sub setPersistentSessionInfo {
 
         return PE_OK unless ( $key and length($key) );
 
-        my $persistentSession =
-          $self->getPersistentSession( $self->_md5hash($key) );
+        my $persistentSession = $self->getPersistentSession($key);
 
         if ($persistentSession) {
             $self->lmLog( "Persistent session found for $key", 'debug' );
             foreach my $k ( keys %{ $persistentSession->data } ) {
 
                 # Do not restore some parameters
-                next if $k =~ /^_session_id$/;
-                next if $k =~ /^_session_kind$/;
+                next if $k =~ /^_(?:utime|session_(?:u?id|kind))$/;
                 $self->lmLog( "Restore persistent parameter $k", 'debug' );
                 $self->{sessionInfo}->{$k} = $persistentSession->data->{$k};
             }
@@ -2359,7 +2448,7 @@ sub store {
     }
 
     # Main session
-    my $session = $self->getApacheSession( $self->{id} );
+    my $session = $self->getApacheSession( $self->{id}, 0, $self->{force} );
     return PE_APACHESESSIONERROR unless ($session);
 
     # Compute unsecure cookie value if needed
@@ -2529,6 +2618,10 @@ sub issuerForAuthUser {
     # Register IssuerDB module in session
     $self->addSessionValue( '_issuerDB', $issuerDBtype, $self->{id} );
 
+    # Update session activity unless for Null IssuerDB
+    $self->updateSession( { '_lastSeen' => time() } )
+      if ( $self->{timeoutActivity} && $issuerDBtype ne 'Null' );
+
     # Call IssuerDB module method
     return $self->SUPER::issuerForAuthUser();
 }
@@ -2566,7 +2659,8 @@ sub autoRedirect {
 
     # Redirection should be made if
     #  - urldc defined
-    if ( $self->{urldc} ) {
+    #  - lmError parameter is not defined
+    if ( $self->{urldc} and !$self->param("lmError") ) {
 
         # Cross-domain mechanism
         if (    $self->{cda}
@@ -2723,7 +2817,7 @@ Lemonldap::NG::Portal::Simple - Base module for building Lemonldap::NG compatibl
     print '<input type="hidden" name="url" value="'.$portal->param('url').'">';
     # Next, login and password
     print 'Login : <input name="user"><br>';
-    print 'Password : <input name="password" type="password" autocomplete="off">';
+    print 'Password : <input name="password" type="password">';
     print '<input type="submit" value="go" />';
     print '</form>';
   }
@@ -2775,7 +2869,7 @@ Lemonldap::Portal::* libraries.
 
 =over
 
-=item * ldapServer: server(s) used to retrive session information and to valid
+=item * ldapServer: server(s) used to retrieve session information and to valid
 credentials (localhost by default). More than one server can be set here
 separated by commas. The servers will be tested in the specifies order.
 To use TLS, set "ldap+tls://server" and to use LDAPS, set "ldaps://server"
@@ -2880,7 +2974,7 @@ if exist. Anonymous bind is provided else.
 
 =head3 search
 
-Retrives the LDAP entry corresponding to the user using $self->{filter}.
+Retrieves the LDAP entry corresponding to the user using $self->{filter}.
 
 =head3 setAuthSessionInfo
 
@@ -2903,7 +2997,7 @@ Does nothing by default.
 =head3 authenticate
 
 Method implemented in Lemonldap::NG::Portal::Auth* modules. By default (ldap),
-authenticates the user by rebinding to the LDAP server using the dn retrived
+authenticates the user by rebinding to the LDAP server using the dn retrieved
 with search() and the password.
 
 =head3 grantSession
@@ -3031,15 +3125,15 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =over
 
-=item Copyright (C) 2006, 2007, 2008, 2009, 2010, 2012 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
+=item Copyright (C) 2005-2016 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
 =item Copyright (C) 2012 by Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
 
-=item Copyright (C) 2012, 2012, 2013 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item Copyright (C) 2012-2013 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
-=item Copyright (C) 2006, 2008, 2009, 2010, 2011, 2012, 2012, 2013 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
+=item Copyright (C) 2006-2016 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
-=item Copyright (C) 2010, 2011 by Thomas Chemineau, E<lt>thomas.chemineau@gmail.comE<gt>
+=item Copyright (C) 2010-2011 by Thomas Chemineau, E<lt>thomas.chemineau@gmail.comE<gt>
 
 =back
 
