@@ -9,7 +9,7 @@ use strict;
 use Lemonldap::NG::Portal::Simple;
 use Lemonldap::NG::Portal::_DBI;    #inherits
 
-our $VERSION = '1.4.0';
+our $VERSION = '1.9.1';
 
 ## @apmethod int userDBInit()
 # Set default values
@@ -124,7 +124,7 @@ sub setGroups {
 # @param value Value to store
 # @return result
 sub setUserDBValue {
-    my ( $self, $key, $value ) = splice @_;
+    my ( $self, $key, $value ) = @_;
 
     # Mandatory attributes
     return 0 unless defined $key;
